import csv
import subprocess
import schedule
import time
from luma.core.interface.serial import i2c, spi, pcf8574
from luma.core.interface.parallel import bitbang_6800
from luma.core.render import canvas
from luma.oled.device import sh1106
from PIL import ImageFont

serial = i2c(port=1, address=0x3C)

device = sh1106(serial)

fontBig = ImageFont.truetype('FreeSans.ttf', 14)
fontNormal = ImageFont.truetype('FreeSans.ttf', 10)
fontSmall = ImageFont.truetype('FreeSans.ttf', 8)

prozesse_dict = {}

def loadProcessInformations():
   command = 'echo "PID;Befehl;CPU Zeit;Speicherverbrauch" > prozesse.csv && top -b -n 1 | awk \'NR > 7 { print $1 ";" $12 ";" $9 ";" $10 }\' >> prozesse.csv'
   subprocess.run(command, shell=True)
   global prozesse_dict
   
   prozesse_dict.clear()

   with open('prozesse.csv', 'r') as file:
       csv_reader = csv.DictReader(file, delimiter=';')
       for row in csv_reader:
            pid = row['PID']
            cpu_zeit = float(row['CPU Zeit'].replace(',', '.'))
            speicherverbrauch = float(row['Speicherverbrauch'].replace(',', '.'))
            if cpu_zeit > 0 and speicherverbrauch > 0:
                befehl = row['Befehl']
                prozesse_dict[pid] = {'Befehl': befehl, 'CPU Zeit': cpu_zeit, 'Speicherverbrauch': speicherverbrauch}
   print(prozesse_dict)

def showProcessEntry(entry, index, total):
   with canvas(device) as draw:
      draw.rectangle(device.bounding_box, outline="white", fill="black")
      draw.text((25, 3), "pyTop Screen", font=fontBig, fill="white")
      draw.text((105, 20), str(index) + "/"+ str(total), font=fontNormal, fill="white")
      draw.text((5, 20), "PID: "+ str(entry), font=fontNormal, fill="white")
      draw.text((5, 30), "Befehl: "+prozesse_dict[entry]['Befehl'], font=fontNormal, fill="white")
      draw.text((5, 40), "CPU Zeit: "+str(prozesse_dict[entry]['CPU Zeit']), font=fontNormal, fill="white")
      draw.text((5, 50), "MEM: "+str(prozesse_dict[entry]['Speicherverbrauch']), font=fontNormal, fill="white")
      

loadProcessInformations()

schedule.every(15).seconds.do(loadProcessInformations)

with canvas(device) as draw:
   draw.rectangle(device.bounding_box, outline="white", fill="black")
   draw.text((25, 3), "pyTop Screen", font=fontBig, fill="white")
   draw.text((5, 20), "Anzeige fuer PID, Befehl,", font=fontNormal, fill="white")
   draw.text((5, 30), "CPU Zeit sowie", font=fontNormal, fill="white")
   draw.text((5, 40), "Speicherverbrauch", font=fontNormal, fill="white")
   
   draw.text((15, 52), "https://draeger-it.blog", font=fontNormal, fill="white")
time.sleep(3)


while True:
    schedule.run_pending()
    total = len(prozesse_dict)
    index = 1
    for entry in prozesse_dict:        
        showProcessEntry(entry, index, total)
        index += 1
        time.sleep(2)
